@@ -32,6 +32,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     const COLLECTION_ERR_STALE               = -3;
     const COLLECTION_ERR_SYNC_REQUIRED       = -4;
     const COLLECTION_ERR_PING_NEED_FULL      = -5;
+    const COLLECTION_ERR_AUTHENTICATION      = -6;
 
     /**
      * The collection data
@@ -525,8 +526,8 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         }
 
         // First try existing, loaded collections.
-        if (!empty($this->_collections[$collection['id']])) {
-            return $this->_collections[$collection['id']]['type'];
+        if (!empty($this->_collections[$id]['type'])) {
+            return $this->_collections[$id]['type'];
         }
 
         // Next look in the SyncCache.
@@ -1104,8 +1105,8 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      * Poll the backend for changes.
      *
      * @param integer $heartbeat  The heartbeat lifetime to wait for changes.
-     * @param integer $interval  The wait interval between poll iterations.
-     * @param array $options  An options array containing any of:
+     * @param integer $interval   The wait interval between poll iterations.
+     * @param array $options      An options array containing any of:
      *   - pingable: (boolean)  Only poll collections with the pingable flag set.
      *                DEFAULT: false
      *
@@ -1229,6 +1230,12 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                         if (!empty($options['pingable'])) {
                             $this->_cache->setPingChangeFlag($id);
                         }
+                    } else {
+                        try {
+                            $this->_as->state->updateSyncStamp();
+                        } catch (Horde_ActiveSync_Exception $e) {
+                            $this->_logger->err($e->getMessage());
+                        }
                     }
                 } catch (Horde_ActiveSync_Exception_StaleState $e) {
                     $this->_logger->notice(sprintf(
@@ -1255,7 +1262,13 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                     // out on their own.
                     $this->resetPingCache();
                     return self::COLLECTION_ERR_FOLDERSYNC_REQUIRED;
-
+                } catch (Horde_Exception_AuthenticationFailure $e) {
+                    // We lost authentication for some reason.
+                    $this->_logger->err(sprintf(
+                        '[%s] Authentication lost during PING!!',
+                        $this->_procid)
+                    );
+                    return self::COLLECTION_ERR_AUTHENTICATION;
                 } catch (Horde_ActiveSync_Exception $e) {
                     $this->_logger->err(sprintf(
                         '[%s] Sync object cannot be configured, throttling: %s',
