@@ -424,7 +424,7 @@ class Horde_ActiveSync_Message_Base
                             if ($map[self::KEY_TYPE] == self::TYPE_DATE || $map[self::KEY_TYPE] == self::TYPE_DATE_DASHES) {
                                 $decoded = $this->_parseDate($decoder->getElementContent());
                             } elseif ($map[self::KEY_TYPE] == self::TYPE_HEX) {
-                                $decoded = self::hex2bin($decoder->getElementContent());
+                                $decoded = self::_hex2bin($decoder->getElementContent());
                             } else {
                                 $class = $map[self::KEY_TYPE];
                                 $subdecoder = new $class(array(
@@ -477,10 +477,15 @@ class Horde_ActiveSync_Message_Base
      * Output is ordered according to $_mapping
      *
      * @param Horde_ActiveSync_Wbxml_Encoder $encoder  The wbxml stream encoder
+     * @throws  Horde_ActiveSync_Exception
      */
     public function encodeStream(Horde_ActiveSync_Wbxml_Encoder &$encoder)
     {
         if (!$this->_preEncodeValidation()) {
+            $this->_logger->err(sprintf(
+                'Pre-encoding validation failed for %s item',
+                get_class($this))
+            );
             throw new Horde_ActiveSync_Exception(sprintf(
                 'Pre-encoding validation failded for %s item',
                 get_class($this))
@@ -621,6 +626,24 @@ class Horde_ActiveSync_Message_Base
         }
     }
 
+    /**
+     * Returns whether or not this message actually contains any data to
+     * send.
+     *
+     * @return boolean  True if message is empty, otherwise false.
+     * @since  2.34.0
+     */
+    public function isEmpty()
+    {
+        foreach ($this->_properties as $value) {
+            if (!empty($value)) {
+                return false;
+            }
+        }
+
+        return true;
+    }
+
     /**
      * Oh yeah. This is beautiful. Exchange outputs date fields differently in
      * calendar items and emails. We could just always send one or the other,
@@ -669,7 +692,7 @@ class Horde_ActiveSync_Message_Base
      *
      * @return string  The binary data
      */
-    private static function hex2bin($data)
+    private static function _hex2bin($data)
     {
         $len = strlen($data);
         $newdata = '';
