@@ -378,33 +378,21 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 return false;
             }
 
-            // Outlook explicitly tells the server to NOT check for server
-            // changes when importing client changes, unlike EVERY OTHER client
-            // out there. This completely screws up many things like conflict
-            // detection since we can't update the sync_ts until we actually
-            // check for changes. So, we need to FORCE a change detection cycle
-            // to be sure we don't screw up state. Any detected changes will be
-            // ignored until the next cycle, utilizing the existing mechanism
-            // for sending MOREITEMS when we will return the previously
-            // selected changes. getchanges defaults to true if it is missing
-            // and the synckey != 0, defaults to true if it is present as an
-            // empty tag. If it is present, but '0' or not present but synckey
-            // is 0 then it defaults to false.
-            if (!isset($collection['getchanges']) && $collection['synckey'] != '0') {
-                $collection['getchanges'] = true;
-            }
-            if (!empty($collection['importedchanges']) && (empty($collection['getchanges']))) {
+            // Clients are allowed to NOT request changes. We still must check
+            // for them since this would otherwise screw up conflict detection
+            // (we can't update sync_ts until we actually check for changes). In
+            // this case, we just don't send the changes back to the client
+            // until the next SYNC that does set GETCHANGES using the
+            // MOREAVAILABLE mechanism.
+            if (!empty($collection['importedchanges']) && empty($collection['getchanges'])) {
                 $forceChanges = true;
                 $collection['getchanges'] = true;
                 $this->_logger->notice(sprintf(
-                    '[%s] Force a GETCHANGES due to incoming changes.',
+                    '[%s] Forcing a GETCHANGES due to incoming changes.',
                     $this->_procid));
             }
 
-            if ($statusCode == self::STATUS_SUCCESS &&
-                (!empty($collection['getchanges']) ||
-                 (!isset($collection['getchanges']) && $collection['synckey'] != '0'))) {
-
+            if ($statusCode == self::STATUS_SUCCESS && !empty($collection['getchanges'])) {
                 try {
                     $changecount = $this->_collections->getCollectionChangeCount();
                 } catch (Horde_ActiveSync_Exception_StaleState $e) {
@@ -428,15 +416,20 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         '[%s] FOLDERSYNC required, collection gone.',
                         $this->_procid));
                     $statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
+                } catch (Horde_Exception_AuthenticationFailure $e) {
+                    $this->_logger->err(sprintf(
+                        '[%s] Lost authentication during SYNC!!',
+                        $this->_procid));
+                    $statusCode = self::STATUS_SERVERERROR;
                 }
             }
 
-            // Get new synckey if needed. We need a new synckey if there were
-            // any changes (incoming or outgoing), if this is during the
-            // initial sync pairing of the collection, or if we received a
-            // SYNC due to changes found during a PING (since those changes
-            // may be changes to items that never even made it to the client in
-            // the first place (See Bug: 12075).
+            // Get new synckey if needed. We need a new synckey if any of the
+            // following are true:
+            //    - There are any changes (incoming or outgoing).
+            //    - This is the initial sync pairing of the collection.
+            //    - We received a SYNC due to changes found during a PING
+            //      (See Bug: 12075).
             if ($statusCode == self::STATUS_SUCCESS &&
                 (!empty($collection['importedchanges']) ||
                 !empty($changecount) ||
@@ -446,7 +439,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $this->_collections->hasPingChangeFlag($id))) {
 
                 try {
-                    $collection['newsynckey'] = $this->_state->getNewSyncKey($collection['synckey']);
+                    $collection['newsynckey'] = $this->_state->getNewSyncKeyWrapper($collection['synckey']);
                     $this->_logger->info(sprintf(
                         '[%s] Old SYNCKEY: %s, New SYNCKEY: %s',
                         $this->_procid,
@@ -485,23 +478,22 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
             if ($statusCode == self::STATUS_SUCCESS) {
 
-                // Send server changes to client
+                // Server changes
                 if ($statusCode == self::STATUS_SUCCESS &&
                     empty($forceChanges) &&
-                    (!empty($collection['getchanges']) ||
-                     (!isset($collection['getchanges']) && !empty($collection['synckey'])))) {
+                    !empty($collection['getchanges'])) {
 
                     $max_windowsize = !empty($pingSettings['maximumwindowsize'])
                         ? min($collection['windowsize'], $pingSettings['maximumwindowsize'])
                         : $collection['windowsize'];
 
                     if (!empty($changecount) &&
-                        (($changecount > $max_windowsize) || $cnt_global + $max_windowsize > $this->_collections->getDefaultWindowSize())) {
+                        (($changecount > $max_windowsize) || $cnt_global + $changecount > $this->_collections->getDefaultWindowSize())) {
                         $this->_logger->info(sprintf(
                             '[%s] Sending MOREAVAILABLE. WINDOWSIZE = %d, $changecount = %d, MAX_REQUEST_WINDOWSIZE = %d, $cnt_global = %d',
                             $this->_procid, $max_windowsize, $changecount, $this->_collections->getDefaultWindowSize(), $cnt_global));
                         $this->_encoder->startTag(Horde_ActiveSync::SYNC_MOREAVAILABLE, false, true);
-                        $over_window = ($cnt_global + $max_windowsize > $this->_collections->getDefaultWindowSize());
+                        $over_window = ($cnt_global + $changecount > $this->_collections->getDefaultWindowSize());
                     }
 
                     if (!empty($changecount)) {
@@ -529,7 +521,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
                     $this->_encoder->startTag(Horde_ActiveSync::SYNC_REPLIES);
 
-                    // Output any SYNC_MODIFY failures
+                    // SYNC_MODIFY failures
                     if (!empty($collection['importfailures'])) {
                         foreach ($collection['importfailures'] as $id => $reason) {
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_MODIFY);
@@ -550,7 +542,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         }
                     }
 
-                    // Output server IDs for new items we received and added from client
+                    // Server IDs for new items we received from client
                     if (!empty($collection['clientids'])) {
                         foreach ($collection['clientids'] as $clientid => $serverid) {
                             if ($serverid) {
@@ -581,7 +573,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                         }
                     }
 
-                    // Output any errors from missing messages in REMOVE requests.
+                    // Errors from missing messages in REMOVE requests.
                     if (!empty($collection['missing'])) {
                         foreach ($collection['missing'] as $uid) {
                             $this->_encoder->startTag(Horde_ActiveSync::SYNC_REMOVE);
@@ -596,7 +588,6 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     }
 
                     if (!empty($collection['fetchids'])) {
-                        // Output any FETCH requests
                         foreach ($collection['fetchids'] as $fetch_id) {
                             try {
                                 $data = $this->_driver->fetch($collection['serverid'], $fetch_id, $collection);
@@ -632,29 +623,29 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                     $this->_encoder->endTag();
                 }
 
-                // Save the sync state for the next time
+                // Save state
                 if (!empty($collection['newsynckey'])) {
-                    if (!empty($sync) || !empty($importer) || $collection['synckey'] == 0)  {
-                        $this->_state->setNewSyncKey($collection['newsynckey']);
-                        $this->_state->save();
-                    } else {
-                        $this->_logger->err(sprintf(
-                            '[%s] Error saving %s - no state information available.',
-                            $this->_procid,
-                            $collection['newsynckey'])
-                        );
-                    }
-
-                    // Do we need to add the new synckey to the syncCache?
-                    if ($collection['newsynckey'] != $collection['synckey']) {
-                        $this->_collections->addConfirmedKey($collection['newsynckey']);
-                    }
+                    $this->_state->setNewSyncKey($collection['newsynckey']);
+                    $this->_state->save();
+                    // Add the new synckey to the syncCache
+                    $this->_collections->addConfirmedKey($collection['newsynckey']);
                     $this->_collections->updateCollection(
                         $collection,
                         array('newsynckey' => true, 'unsetChanges' => true, 'unsetPingChangeFlag' => true)
                     );
+                } elseif (!isset($changes)) {
+                    // See if we could benefit from updating the collection's
+                    // syncStamp value even though there were no changes. If
+                    // $changes is set, we did a looping sync and already took
+                    // care of this.
+                    try {
+                        $this->_state->updateSyncStamp();
+                    } catch (Horde_ActiveSync_Exception $e) {
+                        $this->_logger->err($e->getMessage());
+                    }
                 }
             }
+
             $this->_encoder->endTag();
             $this->_logger->info(sprintf(
                 '[%s] Collection output peak memory usage: %d',
@@ -846,11 +837,13 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
 
                 case Horde_ActiveSync::SYNC_GETCHANGES:
                     // Optional element, but if it's present with an empty value
-                    // it defaults to true. Also, not sent by EAS 14
+                    // it defaults to true.
                     $collection['getchanges'] = $this->_decoder->getElementContent();
                     if ($collection['getchanges'] !== false && !$this->_decoder->getElementEndTag()) {
+                        // Present, has a value, but no closing tag.
                         throw new Horde_ActiveSync_Exception('Protocol Error');
                     } elseif ($collection['getchanges'] === false) {
+                        // Present, but is an empty tag, so defaults to true.
                         $collection['getchanges'] = true;
                     }
                     break;
@@ -882,10 +875,16 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $collection['forcerefresh'] = true;
             }
 
+            // Default value, if missing is TRUE if we have a non-empty synckey,
+            // otherwise FALSE.
+            if (!isset($collection['getchanges'])) {
+                $collection['getchanges'] = !empty($collection['synckey']);
+            }
+
             try {
                 $this->_collections->addCollection($collection);
             } catch (Horde_ActiveSync_Exception_StateGone $e) {
-                $this->_statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
+                $this->_statusCode = self::STATUS_NOTFOUND;
                 $this->_handleError($collection);
                 return false;
             } catch (Horde_ActiveSync_Exception $e) {
@@ -942,9 +941,9 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             $this->_statusCode = self::STATUS_SERVERERROR;
             $this->_handleGlobalSyncError();
             return false;
-        } catch (Horde_ActiveSync_Exception $e) {
+        } catch (Horde_ActiveSync_Exception_FolderGone $e) {
             $this->_statusCode = self::STATUS_FOLDERSYNC_REQUIRED;
-            $this->_handleError($colleciton);
+            $this->_handleError($collection);
             return false;
         } catch (Horde_ActiveSync_Exception $e) {
             $this->_logger->err($e->getMessage());
@@ -1034,47 +1033,54 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
             // Only passed during SYNC_ADD or SYNC_MODIFY
             if (($commandType == Horde_ActiveSync::SYNC_ADD ||
                 $commandType == Horde_ActiveSync::SYNC_MODIFY) &&
-                $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_DATA)) {
-                switch ($collection['class']) {
-                case Horde_ActiveSync::CLASS_EMAIL:
-                    $appdata = Horde_ActiveSync::messageFactory('Mail');
-                    $appdata->decodeStream($this->_decoder);
-                    break;
-                case Horde_ActiveSync::CLASS_CONTACTS:
-                    $appdata = Horde_ActiveSync::messageFactory('Contact');
-                    $appdata->decodeStream($this->_decoder);
-                    break;
-                case Horde_ActiveSync::CLASS_CALENDAR:
-                    $appdata = Horde_ActiveSync::messageFactory('Appointment');
-                    $appdata->decodeStream($this->_decoder);
-                    if (!empty($instanceid) &&
-                        $commandType == Horde_ActiveSync::SYNC_MODIFY) {
-                        // EAS 16.0 sends instanceid/serverid for exceptions.
-                        $appdata->instanceid = $instanceid;
+                ($el = $this->_decoder->getElementStartTag(Horde_ActiveSync::SYNC_DATA))) {
+                if ($this->_decoder->isEmptyElement($el)) {
+                    $this->_logger->err(sprintf(
+                        '[%s] Client sent an empty <Data> element. This is a protocol error, but attempting to ignore.',
+                        $this->_procid)
+                    );
+                } else {
+                    switch ($collection['class']) {
+                    case Horde_ActiveSync::CLASS_EMAIL:
+                        $appdata = Horde_ActiveSync::messageFactory('Mail');
+                        $appdata->decodeStream($this->_decoder);
+                        break;
+                    case Horde_ActiveSync::CLASS_CONTACTS:
+                        $appdata = Horde_ActiveSync::messageFactory('Contact');
+                        $appdata->decodeStream($this->_decoder);
+                        break;
+                    case Horde_ActiveSync::CLASS_CALENDAR:
+                        $appdata = Horde_ActiveSync::messageFactory('Appointment');
+                        $appdata->decodeStream($this->_decoder);
+                        if (!empty($instanceid) &&
+                            $commandType == Horde_ActiveSync::SYNC_MODIFY) {
+                            // EAS 16.0 sends instanceid/serverid for exceptions.
+                            $appdata->instanceid = $instanceid;
+                        }
+                        break;
+                    case Horde_ActiveSync::CLASS_TASKS:
+                        $appdata = Horde_ActiveSync::messageFactory('Task');
+                        $appdata->decodeStream($this->_decoder);
+                        break;
+                    case Horde_ActiveSync::CLASS_NOTES:
+                        $appdata = Horde_ActiveSync::messageFactory('Note');
+                        $appdata->decodeStream($this->_decoder);
+                        break;
+                    case Horde_ActiveSync::CLASS_SMS:
+                        $appdata = Horde_ActiveSync::messageFactory('Mail');
+                        $appdata->decodeStream($this->_decoder);
+                        break;
                     }
-                    break;
-                case Horde_ActiveSync::CLASS_TASKS:
-                    $appdata = Horde_ActiveSync::messageFactory('Task');
-                    $appdata->decodeStream($this->_decoder);
-                    break;
-                case Horde_ActiveSync::CLASS_NOTES:
-                    $appdata = Horde_ActiveSync::messageFactory('Note');
-                    $appdata->decodeStream($this->_decoder);
-                    break;
-                case Horde_ActiveSync::CLASS_SMS:
-                    $appdata = Horde_ActiveSync::messageFactory('Mail');
-                    $appdata->decodeStream($this->_decoder);
-                    break;
-                }
 
-                if (!$this->_decoder->getElementEndTag()) {
-                    // End application data
-                    $this->_statusCode = self::STATUS_PROTERROR;
-                    $this->_handleGlobalSyncError();
-                    return false;
+                    if (!$this->_decoder->getElementEndTag()) {
+                        // End application data
+                        $this->_statusCode = self::STATUS_PROTERROR;
+                        $this->_handleGlobalSyncError();
+                        return false;
+                    }
+                    $appdata->commandType = $commandType;
                 }
             }
-            $appdata->commandType = $commandType;
 
             if (!empty($collection['synckey'])) {
                 switch ($commandType) {
@@ -1212,7 +1218,7 @@ class Horde_ActiveSync_Request_Sync extends Horde_ActiveSync_Request_SyncBase
                 $options['conflict'] = $this->_decoder->getElementContent();
                 if (!$this->_decoder->getElementEndTag()) {
                     $this->_statusCode = self::STATUS_PROTERROR;
-                    $this->_handleError;
+                    $this->_handleError($collection);
                     exit;
                 }
             }
