@@ -146,8 +146,6 @@ abstract class Horde_ActiveSync_State_Base
      * Update the $oldKey syncState to $newKey.
      *
      * @param string $newKey
-     *
-     * @return void
      */
     public function setNewSyncKey($newKey)
     {
@@ -579,6 +577,51 @@ abstract class Horde_ActiveSync_State_Base
         }
     }
 
+    /**
+     * Non-static wrapper for getNewSyncKey.
+     *
+     * @param string $syncKey  The old syncKey
+     *
+     * @return string  The new synckey
+     * @throws Horde_ActiveSync_Exception
+     *
+     * @todo  Remove/replace in H6 with Horde_ActiveSync_SyncKey
+     */
+    public function getNewSyncKeyWrapper($syncKey)
+    {
+        if ($this->checkCollision($newKey = self::getNewSyncKey($syncKey))) {
+            $this->_logger->err(sprintf(
+                '[%s] Found collision when generating synckey %s. Trying again.',
+                $this->_procid, $newKey));
+            return $this->getNewSyncKeyWrapper($synckey);
+        }
+
+        return $newKey;
+    }
+
+    /**
+     * Check for the (rare) possibility of a synckey collision between
+     * collections.
+     *
+     * @param string $syncKey  The synckey to check.
+     *
+     * @return boolean  True if there was a collision.
+     */
+    public function checkCollision($syncKey)
+    {
+        if (!preg_match('/^\{([0-9A-Za-z-]+)\}([0-9]+)$/', $syncKey, $matches)) {
+            return false;
+        }
+        $guid = $matches[1];
+
+        // We only need to check for collisions on the first save.
+        if ($matches[2] != 1) {
+            return false;
+        }
+
+        return $this->_checkCollision($guid);
+    }
+
     /**
      * Gets the new sync key for a specified sync key. You must save the new
      * sync state under this sync key when done sync'ing by calling
@@ -613,7 +656,7 @@ abstract class Horde_ActiveSync_State_Base
      */
     public static function getSyncKeyCounter($syncKey)
     {
-       if (preg_match('/^\{([a-fA-F0-9-]+)\}([0-9]+)$/', $syncKey, $matches)) {
+        if (preg_match('/^\{([a-fA-F0-9-]+)\}([0-9]+)$/', $syncKey, $matches)) {
             $n = $matches[2];
             return $n;
         }
@@ -826,9 +869,11 @@ abstract class Horde_ActiveSync_State_Base
                 $syncKey)
         );
 
-        // Check if synckey is allowed
+        // Check if synckey is allowed. Throw a StateGone exception if it
+        // doesn't match to give the client a chance to reset it's internal
+        // state.
         if (!preg_match('/^\{([0-9A-Za-z-]+)\}([0-9]+)$/', $syncKey, $matches)) {
-            throw new Horde_ActiveSync_Exception('Invalid sync key');
+            throw new Horde_ActiveSync_Exception_StateGone('Invalid synckey');
         }
         $this->_syncKey = $syncKey;
 
@@ -836,7 +881,7 @@ abstract class Horde_ActiveSync_State_Base
         $this->_gc($syncKey);
 
         // Load the state
-        $this->_loadState($type);
+        $this->_loadState();
     }
 
     /**
@@ -860,12 +905,9 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Load the state represented by $syncKey from storage.
      *
-     * @param string $type       The type of state a
-     *                           Horde_ActiveSync::REQUEST_TYPE constant.
-     *
      * @throws Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_StateGone
      */
-    protected function _loadState($type)
+    protected function _loadState()
     {
         throw new Horde_ActiveSync_Exception('Must be implemented in concrete class.');
     }
@@ -902,6 +944,15 @@ abstract class Horde_ActiveSync_State_Base
         throw new Horde_ActiveSync_Exception('Must be implemented in concrete class.');
     }
 
+    /**
+     * Update the syncStamp in the collection state, outside of any other changes.
+     * Used to prevent extremely large differences in syncStamps for clients
+     * and collections that don't often have changes.
+     */
+    public function updateSyncStamp()
+    {
+        throw new Horde_ActiveSync_Exception('Not supported in this state driver.');
+    }
 
     /**
      * Save the current syncstate to storage
@@ -962,7 +1013,7 @@ abstract class Horde_ActiveSync_State_Base
     /**
      * Obtain the device object.
      *
-     * @param object $device
+     * @param string $devId   The device id to obtain
      * @param string $user
      * @param array  $params  Additional parameters:
      *   - force: (boolean)  If true, reload the device info even if it's
