@@ -118,6 +118,14 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     protected $_syncCacheTable;
 
+    /**
+     * When there are no changes found in a collection, but the difference in
+     * syncStamp values is more than this threshold, the syncStamp is updated
+     * in the collection state without modifying the synckey or anyother
+     * state.
+     */
+    const SYNCSTAMP_UPDATE_THRESHOLD = 30000;
+
     /**
      * Const'r
      *
@@ -217,17 +225,20 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     /**
      * Load the state represented by $syncKey from storage.
      *
-     * @param string $type       The type of state a
-     *                           Horde_ActiveSync::REQUEST_TYPE constant.
-     *
      * @throws Horde_ActiveSync_Exception, Horde_ActiveSync_Exception_StateGone
      */
-    protected function _loadState($type)
+    protected function _loadState()
     {
         // Load the previous syncState from storage
+        $sql = 'SELECT sync_data, sync_devid, sync_mod, sync_pending FROM '
+            . $this->_syncStateTable . ' WHERE sync_key = ?';
+        $values = array($this->_syncKey);
+        if (!empty($this->_collection['id'])) {
+            $sql .= ' AND sync_folderid = ?';
+            $values[] = $this->_collection['id'];
+        }
         try {
-            $results = $this->_db->selectOne('SELECT sync_data, sync_devid, sync_mod, sync_pending FROM '
-                . $this->_syncStateTable . ' WHERE sync_key = ?', array($this->_syncKey));
+            $results = $this->_db->selectOne($sql, $values);
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err(sprintf(
                 '[%s] %s',
@@ -245,18 +256,17 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             throw new Horde_ActiveSync_Exception_StateGone();
         }
 
-        $this->_loadStateFromResults($results, $type);
+        $this->_loadStateFromResults($results);
     }
 
     /**
      * Actually load the state data into the object from the query results.
      *
      * @param array $results  The results array from the state query.
-     * @param string $type    The type of request we are handling.
      *
      * @throws Horde_ActiveSync_Exception_StateGone
      */
-    protected function _loadStateFromResults($results, $type = Horde_ActiveSync::REQUEST_TYPE_SYNC)
+    protected function _loadStateFromResults($results)
     {
         // Load the last known sync time for this collection
         $this->_lastSyncStamp = !empty($results['sync_mod'])
@@ -281,13 +291,13 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         $data = unserialize($columns['sync_data']->binaryToString($results['sync_data']));
         $pending = unserialize($results['sync_pending']);
 
-        if ($type == Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC) {
+        if ($this->_type == Horde_ActiveSync::REQUEST_TYPE_FOLDERSYNC) {
             $this->_folder = ($data !== false) ? $data : array();
             $this->_logger->info(
                 sprintf('[%s] Loading FOLDERSYNC state containing %d folders.',
                 $this->_procid,
                 count($this->_folder)));
-        } elseif ($type == Horde_ActiveSync::REQUEST_TYPE_SYNC) {
+        } elseif ($this->_type == Horde_ActiveSync::REQUEST_TYPE_SYNC) {
             // @TODO: This shouldn't default to an empty folder object,
             // if we don't have the data, it's an exception.
             $this->_folder = ($data !== false
@@ -306,6 +316,43 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
     }
 
+
+    /**
+     * Update the syncStamp in the collection state, outside of any other changes.
+     * Used to prevent extremely large differences in syncStamps for clients
+     * and collections that don't often have changes and the backend server
+     * doesn't keep separate syncStamp values per collection.
+     *
+     * @throws  Horde_ActiveSync_Exception
+     */
+    public function updateSyncStamp()
+    {
+        if (($this->_thisSyncStamp - $this->_lastSyncStamp) >= self::SYNCSTAMP_UPDATE_THRESHOLD) {
+            $this->_logger->info(sprintf(
+                '[%s] Updating sync_mod value from %s to %s without changes.',
+                $this->_procid,
+                $this->_lastSyncStamp,
+                $this->_thisSyncStamp)
+            );
+            $sql = 'UPDATE ' . $this->_syncStateTable . ' SET sync_mod = ?'
+                . ' WHERE sync_mod = ? AND sync_key = ? AND sync_user = ? AND sync_folderid = ?';
+            try {
+                $this->_db->update(
+                    $sql,
+                    array(
+                        $this->_thisSyncStamp,
+                        $this->_lastSyncStamp,
+                        $this->_syncKey,
+                        $this->_deviceInfo->user,
+                        $this->_collection['id']
+                    )
+                );
+            } catch (Horde_Db_Exception $e) {
+                throw new Horde_ActiveSync_Exception($e);
+            }
+        }
+    }
+
     /**
      * Save the current state to storage
      *
@@ -326,7 +373,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
 
         // If we are setting the first synckey iteration, do not save the
-        // timestamp, otherwise we will never get the initial set of data.
+        // syncstamp/mod, otherwise we will never get the initial set of data.
         $params = array(
             'sync_key' => $this->_syncKey,
             'sync_data' => new Horde_Db_Value_Binary($data),
@@ -447,13 +494,20 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                             . ' sync_folderid, sync_user, sync_read)'
                             . ' VALUES (?, ?, ?, ?, ?, ?)';
                         $flag_value = !empty($change['flags']['read']);
-                    } else {
+                    } elseif (isset($change['flags']['flagged'])) {
                         $sql = 'INSERT INTO ' . $this->_syncMailMapTable
                             . ' (message_uid, sync_key, sync_devid,'
                             . ' sync_folderid, sync_user, sync_flagged)'
                             . ' VALUES (?, ?, ?, ?, ?, ?)';
                         $flag_value = !empty($change['flags']['flagged']);
                     }
+                    if (isset($change['categories'])) {
+                         $sql = 'INSERT INTO ' . $this->_syncMailMapTable
+                            . ' (message_uid, sync_key, sync_devid,'
+                            . ' sync_folderid, sync_user, sync_category)'
+                            . ' VALUES (?, ?, ?, ?, ?, ?)';
+                        $flag_value = md5(implode('', $change['categories']));
+                    }
                     break;
                 case Horde_ActiveSync::CHANGE_TYPE_DELETE:
                     $sql = 'INSERT INTO ' . $this->_syncMailMapTable
@@ -499,10 +553,10 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                 throw new Horde_ActiveSync_Exception($e);
             }
         } else {
-            // We are sending server changes; $this->_changes will contain all
-            // changes so we need to track which ones are sent since not all
-            // may be sent. We need to store the leftovers for sending next
-            // request.
+            // We are sending server changes; $this->_changes contains changes.
+            // We need to track which ones are sent since not all may be sent.
+            // Store the leftovers for sending next request via MOREAVAILABLE.
+            //
             // @todo FIX BC HACK for differing data structures when sending
             // initial change set.
             foreach ($this->_changes as $key => $value) {
@@ -893,7 +947,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      *   - id:      (string)  When removing device state, restrict ro removing data
      *                        only for this collection.
      *
-     * @throws Horde_ActiveSyncException
+     * @throws Horde_ActiveSync_Exception
      */
     public function removeState(array $options)
     {
@@ -1219,13 +1273,16 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     protected function _getPIMChangeTS(array $changes)
     {
         $sql = 'SELECT message_uid, MAX(sync_modtime) FROM ' . $this->_syncMapTable
-            . ' WHERE sync_devid = ? AND sync_user = ? AND sync_key IN (?, ?) ';
+            . ' WHERE sync_devid = ? AND sync_user = ? AND sync_folderid = ? AND sync_key IN (?, ?) ';
 
         // Get the allowed synckeys to include.
         $uuid = self::getSyncKeyUid($this->_syncKey);
         $cnt = self::getSyncKeyCounter($this->_syncKey);
-        $values = array($this->_deviceInfo->id, $this->_deviceInfo->user);
-        foreach (array($uuid . $cnt, $uuid . ($cnt - 1)) as $v) {
+        $values = array($this->_deviceInfo->id,
+            $this->_deviceInfo->user,
+            $this->_collection['serverid']
+        );
+        foreach (array($this->_syncKey, $uuid . ($cnt - 1)) as $v) {
             $values[] = $v;
         }
 
@@ -1288,7 +1345,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
     protected function _getMailMapChanges(array $changes)
     {
         $sql = 'SELECT message_uid, sync_read, sync_flagged, sync_deleted,'
-            . 'sync_changed FROM ' . $this->_syncMailMapTable
+            . 'sync_changed, sync_category FROM ' . $this->_syncMailMapTable
             . ' WHERE sync_folderid = ? AND sync_devid = ?'
             . ' AND sync_user = ? AND message_uid IN '
             . '(' . implode(',', array_fill(0, count($changes), '?')) . ')';
@@ -1317,7 +1374,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                     case Horde_ActiveSync::CHANGE_TYPE_FLAGS:
                         $results[$row['message_uid']][$change['type']] =
                             (!is_null($row['sync_read']) && $row['sync_read'] == $change['flags']['read']) ||
-                            (!is_null($row['sync_flagged'] && $row['sync_flagged'] == $change['flags']['flagged']));
+                            (!is_null($row['sync_flagged']) && $row['sync_flagged'] == $change['flags']['flagged']) ||
+                            (!is_null($row['sync_category']) && $row['sync_category'] == md5(implode('', $change['categories'])));
                         continue 3;
                     case Horde_ActiveSync::CHANGE_TYPE_DELETE:
                         $results[$row['message_uid']][$change['type']] =
@@ -1335,6 +1393,32 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         return $results;
     }
 
+    /**
+     * Check for the (rare) possibility of a synckey collision between
+     * collections.
+     *
+     * @param string $guid The GUID portion of the synckey to check.
+     *
+     * @return boolean  true if there was a collision.
+     */
+    protected function _checkCollision($guid)
+    {
+        $sql = 'SELECT COUNT(sync_key) FROM ' . $this->_syncStateTable
+            . ' WHERE sync_devid = ? AND sync_folderid != ? AND sync_key LIKE ?';
+        $values = array($this->_deviceInfo->id, $this->_collection['id'], '{' . $guid . '}%');
+
+        try {
+            return (boolean)$this->_db->selectValue($sql, $values);
+        } catch (Horde_Db_Exception $e) {
+            $this->_logger->err(sprintf(
+                '[%s] %s',
+                $this->_procid,
+                $e->getMessage())
+            );
+            throw new Horde_ActiveSync_Exception($e);
+        }
+    }
+
     /**
      * Garbage collector - clean up from previous sync requests.
      *
@@ -1354,12 +1438,13 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         // ensures that we can still respond to SYNC requests for the previous
         // key if the client never received the new key in a SYNC response.
         $sql = 'SELECT sync_key FROM ' . $this->_syncStateTable
-            . ' WHERE sync_devid = ? AND sync_folderid = ?';
+            . ' WHERE sync_devid = ? AND sync_folderid = ? AND sync_user = ?';
         $values = array(
             $this->_deviceInfo->id,
             !empty($this->_collection['id'])
                 ? $this->_collection['id']
-                : Horde_ActiveSync::CHANGE_TYPE_FOLDERSYNC);
+                : Horde_ActiveSync::CHANGE_TYPE_FOLDERSYNC,
+            $this->_deviceInfo->user);
 
         try {
             $results = $this->_db->selectAll($sql, $values);
